@@ -16,7 +16,7 @@ use std::{
     hash::Hash,
     mem,
     num::{NonZeroU8, NonZeroUsize},
-    ops::{Add, AddAssign, Deref, DerefMut},
+    ops::{Add, AddAssign, Deref, DerefMut, Not, Sub, SubAssign},
 };
 
 #[cfg(not(feature = "no_float"))]
@@ -40,7 +40,7 @@ pub enum FnAccess {
     Private,
 }
 
-/// _(INTERNALS)_ A type containing information on a scripted function.
+/// _(internals)_ A type containing information on a scripted function.
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -68,7 +68,7 @@ pub struct ScriptFnDef {
     /// Not available under `no_closure`.
     #[cfg(not(feature = "no_closure"))]
     pub externals: std::collections::BTreeSet<Identifier>,
-    /// _(METADATA)_ Function doc-comments (if any).
+    /// _(metadata)_ Function doc-comments (if any).
     /// Exported under the `metadata` feature only.
     ///
     /// Not available under `no_function`.
@@ -78,7 +78,6 @@ pub struct ScriptFnDef {
 }
 
 impl fmt::Display for ScriptFnDef {
-    #[inline(always)]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
             f,
@@ -105,7 +104,7 @@ impl fmt::Display for ScriptFnDef {
 #[cfg(not(feature = "no_function"))]
 #[derive(Debug, Eq, PartialEq, Clone, Hash)]
 pub struct ScriptFnMetadata<'a> {
-    /// _(METADATA)_ Function doc-comments (if any).
+    /// _(metadata)_ Function doc-comments (if any).
     /// Exported under the `metadata` feature only.
     ///
     /// Not available under `no_function`.
@@ -129,7 +128,6 @@ pub struct ScriptFnMetadata<'a> {
 
 #[cfg(not(feature = "no_function"))]
 impl fmt::Display for ScriptFnMetadata<'_> {
-    #[inline(always)]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
             f,
@@ -149,16 +147,16 @@ impl fmt::Display for ScriptFnMetadata<'_> {
 }
 
 #[cfg(not(feature = "no_function"))]
-impl<'a> Into<ScriptFnMetadata<'a>> for &'a ScriptFnDef {
-    #[inline(always)]
-    fn into(self) -> ScriptFnMetadata<'a> {
-        ScriptFnMetadata {
+impl<'a> From<&'a ScriptFnDef> for ScriptFnMetadata<'a> {
+    #[inline]
+    fn from(value: &'a ScriptFnDef) -> Self {
+        Self {
             #[cfg(not(feature = "no_function"))]
             #[cfg(feature = "metadata")]
-            comments: self.comments.iter().map(|s| s.as_str()).collect(),
-            access: self.access,
-            name: &self.name,
-            params: self.params.iter().map(|s| s.as_str()).collect(),
+            comments: value.comments.iter().map(|s| s.as_str()).collect(),
+            access: value.access,
+            name: &value.name,
+            params: value.params.iter().map(|s| s.as_str()).collect(),
         }
     }
 }
@@ -215,7 +213,7 @@ impl Default for AST {
 
 impl AST {
     /// Create a new [`AST`].
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub fn new(
         statements: impl IntoIterator<Item = Stmt>,
@@ -223,7 +221,7 @@ impl AST {
     ) -> Self {
         Self {
             source: None,
-            body: StmtBlock::new(statements.into_iter().collect(), Position::NONE),
+            body: StmtBlock::new(statements, Position::NONE),
             functions: functions.into(),
             #[cfg(not(feature = "no_module"))]
             resolver: None,
@@ -239,7 +237,7 @@ impl AST {
     ) -> Self {
         Self {
             source: Some(source.into()),
-            body: StmtBlock::new(statements.into_iter().collect(), Position::NONE),
+            body: StmtBlock::new(statements, Position::NONE),
             functions: functions.into(),
             #[cfg(not(feature = "no_module"))]
             resolver: None,
@@ -258,7 +256,7 @@ impl AST {
         self.source.as_ref()
     }
     /// Set the source.
-    #[inline(always)]
+    #[inline]
     pub fn set_source(&mut self, source: impl Into<Identifier>) -> &mut Self {
         let source = source.into();
         Shared::get_mut(&mut self.functions)
@@ -280,7 +278,7 @@ impl AST {
     pub(crate) fn statements(&self) -> &[Stmt] {
         &self.body.0
     }
-    /// _(INTERNALS)_ Get the statements.
+    /// _(internals)_ Get the statements.
     /// Exported under the `internals` feature only.
     #[cfg(feature = "internals")]
     #[deprecated = "this method is volatile and may change"]
@@ -305,7 +303,7 @@ impl AST {
     pub(crate) fn shared_lib(&self) -> Shared<Module> {
         self.functions.clone()
     }
-    /// _(INTERNALS)_ Get the internal shared [`Module`] containing all script-defined functions.
+    /// _(internals)_ Get the internal shared [`Module`] containing all script-defined functions.
     /// Exported under the `internals` feature only.
     ///
     /// Not available under `no_function` or `no_module`.
@@ -325,7 +323,7 @@ impl AST {
     pub(crate) fn lib(&self) -> &Module {
         &self.functions
     }
-    /// _(INTERNALS)_ Get the internal [`Module`] containing all script-defined functions.
+    /// _(internals)_ Get the internal [`Module`] containing all script-defined functions.
     /// Exported under the `internals` feature only.
     ///
     /// Not available under `no_function`.
@@ -346,7 +344,7 @@ impl AST {
     ) -> Option<Shared<crate::module::resolvers::StaticModuleResolver>> {
         self.resolver.clone()
     }
-    /// _(INTERNALS)_ Get the embedded [module resolver][crate::ModuleResolver].
+    /// _(internals)_ Get the embedded [module resolver][crate::ModuleResolver].
     /// Exported under the `internals` feature only.
     ///
     /// Not available under `no_module`.
@@ -386,7 +384,7 @@ impl AST {
     ///
     /// This operation is cheap because functions are shared.
     #[cfg(not(feature = "no_function"))]
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub fn clone_functions_only_filtered(
         &self,
@@ -657,7 +655,7 @@ impl AST {
     /// # Ok(())
     /// # }
     /// ```
-    #[inline(always)]
+    #[inline]
     pub fn combine_filtered(
         &mut self,
         other: Self,
@@ -696,7 +694,7 @@ impl AST {
     /// # }
     /// ```
     #[cfg(not(feature = "no_function"))]
-    #[inline(always)]
+    #[inline]
     pub fn retain_functions(
         &mut self,
         filter: impl Fn(FnNamespace, FnAccess, &str, usize) -> bool,
@@ -712,7 +710,6 @@ impl AST {
     #[cfg(not(feature = "no_function"))]
     #[cfg(not(feature = "no_module"))]
     #[inline(always)]
-    #[must_use]
     pub(crate) fn iter_fn_def(&self) -> impl Iterator<Item = &ScriptFnDef> {
         self.functions
             .iter_script_fn()
@@ -723,7 +720,6 @@ impl AST {
     /// Not available under `no_function`.
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
-    #[must_use]
     pub fn iter_functions<'a>(&'a self) -> impl Iterator<Item = ScriptFnMetadata> + 'a {
         self.functions
             .iter_script_fn()
@@ -748,7 +744,7 @@ impl AST {
     /// Return `false` from the callback to terminate the walk.
     #[cfg(not(feature = "internals"))]
     #[cfg(not(feature = "no_module"))]
-    #[inline(always)]
+    #[inline]
     pub(crate) fn walk(&self, on_node: &mut impl FnMut(&[ASTNode]) -> bool) -> bool {
         let path = &mut Default::default();
 
@@ -766,11 +762,11 @@ impl AST {
 
         true
     }
-    /// _(INTERNALS)_ Recursively walk the [`AST`], including function bodies (if any).
+    /// _(internals)_ Recursively walk the [`AST`], including function bodies (if any).
     /// Return `false` from the callback to terminate the walk.
     /// Exported under the `internals` feature only.
     #[cfg(feature = "internals")]
-    #[inline(always)]
+    #[inline]
     pub fn walk(&self, on_node: &mut impl FnMut(&[ASTNode]) -> bool) -> bool {
         let path = &mut Default::default();
 
@@ -820,7 +816,7 @@ impl AsRef<Module> for AST {
     }
 }
 
-/// _(INTERNALS)_ An identifier containing a name and a [position][Position].
+/// _(internals)_ An identifier containing a name and a [position][Position].
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -835,14 +831,13 @@ pub struct Ident {
 }
 
 impl fmt::Debug for Ident {
-    #[inline(always)]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "{:?}", self.name)?;
         self.pos.debug_print(f)
     }
 }
 
-/// _(INTERNALS)_ A type encapsulating the mode of a `return`/`throw` statement.
+/// _(internals)_ A type encapsulating the mode of a `return`/`throw` statement.
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -856,7 +851,7 @@ pub enum ReturnType {
     Exception,
 }
 
-/// _(INTERNALS)_ An [`AST`] node, consisting of either an [`Expr`] or a [`Stmt`].
+/// _(internals)_ An [`AST`] node, consisting of either an [`Expr`] or a [`Stmt`].
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -880,7 +875,7 @@ impl<'a> From<&'a Expr> for ASTNode<'a> {
     }
 }
 
-/// _(INTERNALS)_ A statements block.
+/// _(internals)_ A statements block.
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -892,7 +887,8 @@ pub struct StmtBlock(StaticVec<Stmt>, Position);
 impl StmtBlock {
     /// Create a new [`StmtBlock`].
     #[must_use]
-    pub fn new(mut statements: StaticVec<Stmt>, pos: Position) -> Self {
+    pub fn new(statements: impl IntoIterator<Item = Stmt>, pos: Position) -> Self {
+        let mut statements: StaticVec<_> = statements.into_iter().collect();
         statements.shrink_to_fit();
         Self(statements, pos)
     }
@@ -939,7 +935,6 @@ impl DerefMut for StmtBlock {
 }
 
 impl fmt::Debug for StmtBlock {
-    #[inline(always)]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str("Block")?;
         fmt::Debug::fmt(&self.0, f)?;
@@ -955,7 +950,112 @@ impl From<StmtBlock> for Stmt {
     }
 }
 
-/// _(INTERNALS)_ A statement.
+/// A type that holds a configuration option with bit-flags.
+/// Exported under the `internals` feature only.
+#[derive(PartialEq, Eq, Copy, Clone, Hash, Default)]
+pub struct OptionFlags(u8);
+
+impl OptionFlags {
+    /// Does this [`BitOptions`] contain a particular option flag?
+    #[inline(always)]
+    #[must_use]
+    pub const fn contains(self, flag: Self) -> bool {
+        self.0 & flag.0 != 0
+    }
+}
+
+impl Not for OptionFlags {
+    type Output = Self;
+
+    #[inline(always)]
+    fn not(self) -> Self::Output {
+        Self(!self.0)
+    }
+}
+
+impl Add for OptionFlags {
+    type Output = Self;
+
+    #[inline(always)]
+    fn add(self, rhs: Self) -> Self::Output {
+        Self(self.0 | rhs.0)
+    }
+}
+
+impl AddAssign for OptionFlags {
+    #[inline(always)]
+    fn add_assign(&mut self, rhs: Self) {
+        self.0 |= rhs.0
+    }
+}
+
+impl Sub for OptionFlags {
+    type Output = Self;
+
+    #[inline(always)]
+    fn sub(self, rhs: Self) -> Self::Output {
+        Self(self.0 & !rhs.0)
+    }
+}
+
+impl SubAssign for OptionFlags {
+    #[inline(always)]
+    fn sub_assign(&mut self, rhs: Self) {
+        self.0 &= !rhs.0
+    }
+}
+
+/// Option bit-flags for [`AST`] nodes.
+#[allow(non_snake_case)]
+pub mod AST_OPTION_FLAGS {
+    use super::OptionFlags;
+
+    /// _(internals)_ No options for the [`AST`][crate::AST] node.
+    /// Exported under the `internals` feature only.
+    pub const AST_OPTION_NONE: OptionFlags = OptionFlags(0b0000_0000);
+    /// _(internals)_ The [`AST`][crate::AST] node is constant.
+    /// Exported under the `internals` feature only.
+    pub const AST_OPTION_CONSTANT: OptionFlags = OptionFlags(0b0000_0001);
+    /// _(internals)_ The [`AST`][crate::AST] node is exported.
+    /// Exported under the `internals` feature only.
+    pub const AST_OPTION_EXPORTED: OptionFlags = OptionFlags(0b0000_0010);
+    /// _(internals)_ The [`AST`][crate::AST] node is in negated mode.
+    /// Exported under the `internals` feature only.
+    pub const AST_OPTION_NEGATED: OptionFlags = OptionFlags(0b0000_0100);
+
+    impl std::fmt::Debug for OptionFlags {
+        fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+            fn write_option(
+                options: &OptionFlags,
+                f: &mut std::fmt::Formatter<'_>,
+                num_flags: &mut usize,
+                flag: OptionFlags,
+                name: &str,
+            ) -> std::fmt::Result {
+                if options.contains(flag) {
+                    if *num_flags > 0 {
+                        f.write_str("+")?;
+                    }
+                    f.write_str(name)?;
+                    *num_flags += 1;
+                }
+                Ok(())
+            }
+
+            let num_flags = &mut 0;
+
+            f.write_str("(")?;
+            write_option(self, f, num_flags, AST_OPTION_CONSTANT, "Constant")?;
+            write_option(self, f, num_flags, AST_OPTION_EXPORTED, "Exported")?;
+            write_option(self, f, num_flags, AST_OPTION_NEGATED, "Negated")?;
+            f.write_str(")")?;
+
+            Ok(())
+        }
+    }
+}
+
+/// _(internals)_ A statement.
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -973,16 +1073,26 @@ pub enum Stmt {
         Box<(BTreeMap<u64, Box<(Option<Expr>, StmtBlock)>>, StmtBlock)>,
         Position,
     ),
-    /// `while` expr `{` stmt `}`
+    /// `while` expr `{` stmt `}` | `loop` `{` stmt `}`
+    ///
+    /// If the guard expression is [`UNIT`][Expr::Unit], then it is a `loop` statement.
     While(Expr, Box<StmtBlock>, Position),
     /// `do` `{` stmt `}` `while`|`until` expr
-    Do(Box<StmtBlock>, Expr, bool, Position),
+    ///
+    /// ### Option Flags
+    ///
+    /// * [`AST_FLAG_NONE`][AST_FLAGS::AST_FLAG_NONE] = `while`
+    /// * [`AST_FLAG_NEGATED`][AST_FLAGS::AST_FLAG_NEGATED] = `until`
+    Do(Box<StmtBlock>, Expr, OptionFlags, Position),
     /// `for` `(` id `,` counter `)` `in` expr `{` stmt `}`
     For(Expr, Box<(Ident, Option<Ident>, StmtBlock)>, Position),
-    /// \[`export`\] `let` id `=` expr
-    Let(Expr, Box<Ident>, bool, Position),
-    /// \[`export`\] `const` id `=` expr
-    Const(Expr, Box<Ident>, bool, Position),
+    /// \[`export`\] `let`|`const` id `=` expr
+    ///
+    /// ### Option Flags
+    ///
+    /// * [`AST_FLAG_EXPORTED`][AST_FLAGS::AST_FLAG_EXPORTED] = `export`
+    /// * [`AST_FLAG_CONSTANT`][AST_FLAGS::AST_FLAG_CONSTANT] = `const`
+    Var(Expr, Box<Ident>, OptionFlags, Position),
     /// expr op`=` expr
     Assignment(Box<(Expr, Option<OpAssignment<'static>>, Expr)>, Position),
     /// func `(` expr `,` ... `)`
@@ -1027,7 +1137,7 @@ impl Default for Stmt {
 }
 
 impl From<Stmt> for StmtBlock {
-    #[inline(always)]
+    #[inline]
     fn from(stmt: Stmt) -> Self {
         match stmt {
             Stmt::Block(mut block, pos) => Self(block.iter_mut().map(mem::take).collect(), pos),
@@ -1045,10 +1155,7 @@ impl Stmt {
     #[inline(always)]
     #[must_use]
     pub const fn is_noop(&self) -> bool {
-        match self {
-            Self::Noop(_) => true,
-            _ => false,
-        }
+        matches!(self, Self::Noop(_))
     }
     /// Get the [position][Position] of this statement.
     #[must_use]
@@ -1066,8 +1173,7 @@ impl Stmt {
             | Self::Do(_, _, _, pos)
             | Self::For(_, _, pos)
             | Self::Return(_, _, pos)
-            | Self::Let(_, _, _, pos)
-            | Self::Const(_, _, _, pos)
+            | Self::Var(_, _, _, pos)
             | Self::TryCatch(_, pos) => *pos,
 
             Self::Expr(x) => x.position(),
@@ -1096,8 +1202,7 @@ impl Stmt {
             | Self::Do(_, _, _, pos)
             | Self::For(_, _, pos)
             | Self::Return(_, _, pos)
-            | Self::Let(_, _, _, pos)
-            | Self::Const(_, _, _, pos)
+            | Self::Var(_, _, _, pos)
             | Self::TryCatch(_, pos) => *pos = new_pos,
 
             Self::Expr(x) => {
@@ -1131,8 +1236,7 @@ impl Stmt {
             | Self::For(_, _, _)
             | Self::TryCatch(_, _) => false,
 
-            Self::Let(_, _, _, _)
-            | Self::Const(_, _, _, _)
+            Self::Var(_, _, _, _)
             | Self::Assignment(_, _)
             | Self::Continue(_)
             | Self::Break(_)
@@ -1159,8 +1263,7 @@ impl Stmt {
             // A No-op requires a semicolon in order to know it is an empty statement!
             Self::Noop(_) => false,
 
-            Self::Let(_, _, _, _)
-            | Self::Const(_, _, _, _)
+            Self::Var(_, _, _, _)
             | Self::Assignment(_, _)
             | Self::FnCall(_, _)
             | Self::Expr(_)
@@ -1197,14 +1300,23 @@ impl Stmt {
                     })
                     && (x.1).0.iter().all(Stmt::is_pure)
             }
-            Self::While(condition, block, _) | Self::Do(block, condition, _, _) => {
-                condition.is_pure() && block.0.iter().all(Stmt::is_pure)
+
+            // Loops that exit can be pure because it can never be infinite.
+            Self::While(Expr::BoolConstant(false, _), _, _) => true,
+            Self::Do(body, Expr::BoolConstant(x, _), options, _)
+                if *x == options.contains(AST_OPTION_FLAGS::AST_OPTION_NEGATED) =>
+            {
+                body.iter().all(Stmt::is_pure)
             }
+
+            // Loops are never pure since they can be infinite - and that's a side effect.
+            Self::While(_, _, _) | Self::Do(_, _, _, _) => false,
+
+            // For loops can be pure because if the iterable is pure, it is finite,
+            // so infinite loops can never occur.
             Self::For(iterable, x, _) => iterable.is_pure() && (x.2).0.iter().all(Stmt::is_pure),
-            Self::Let(_, _, _, _)
-            | Self::Const(_, _, _, _)
-            | Self::Assignment(_, _)
-            | Self::FnCall(_, _) => false,
+
+            Self::Var(_, _, _, _) | Self::Assignment(_, _) | Self::FnCall(_, _) => false,
             Self::Block(block, _) => block.iter().all(|stmt| stmt.is_pure()),
             Self::Continue(_) | Self::Break(_) | Self::Return(_, _, _) => false,
             Self::TryCatch(x, _) => {
@@ -1226,11 +1338,11 @@ impl Stmt {
     ///
     /// Only variable declarations (i.e. `let` and `const`) and `import`/`export` statements
     /// are internally pure.
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub fn is_internally_pure(&self) -> bool {
         match self {
-            Self::Let(expr, _, _, _) | Self::Const(expr, _, _, _) => expr.is_pure(),
+            Self::Var(expr, _, _, _) => expr.is_pure(),
 
             #[cfg(not(feature = "no_module"))]
             Self::Import(expr, _, _) => expr.is_pure(),
@@ -1245,7 +1357,7 @@ impl Stmt {
     /// Currently this is only true for `return`, `throw`, `break` and `continue`.
     ///
     /// All statements following this statement will essentially be dead code.
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub const fn is_control_flow_break(&self) -> bool {
         match self {
@@ -1268,7 +1380,7 @@ impl Stmt {
         }
 
         match self {
-            Self::Let(e, _, _, _) | Self::Const(e, _, _, _) => {
+            Self::Var(e, _, _, _) => {
                 if !e.walk(path, on_node) {
                     return false;
                 }
@@ -1383,7 +1495,7 @@ impl Stmt {
     }
 }
 
-/// _(INTERNALS)_ A custom syntax expression.
+/// _(internals)_ A custom syntax expression.
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -1399,7 +1511,7 @@ pub struct CustomExpr {
     pub tokens: StaticVec<Identifier>,
 }
 
-/// _(INTERNALS)_ A binary expression.
+/// _(internals)_ A binary expression.
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -1413,7 +1525,7 @@ pub struct BinaryExpr {
     pub rhs: Expr,
 }
 
-/// _(INTERNALS)_ An op-assignment operator.
+/// _(internals)_ An op-assignment operator.
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -1451,7 +1563,7 @@ impl OpAssignment<'_> {
     }
 }
 
-/// _(INTERNALS)_ An set of function call hashes.
+/// _(internals)_ An set of function call hashes.
 /// Exported under the `internals` feature only.
 ///
 /// Two separate hashes are pre-calculated because of the following pattern:
@@ -1538,7 +1650,7 @@ impl FnCallHashes {
     }
 }
 
-/// _(INTERNALS)_ A function call.
+/// _(internals)_ A function call.
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -1634,7 +1746,6 @@ impl<F: Float + fmt::Debug> fmt::Debug for FloatWrapper<F> {
 
 #[cfg(not(feature = "no_float"))]
 impl<F: Float + fmt::Display + fmt::LowerExp + From<f32>> fmt::Display for FloatWrapper<F> {
-    #[inline(always)]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let abs = self.0.abs();
         if abs > Self::MAX_NATURAL_FLOAT_FOR_DISPLAY.into()
@@ -1690,12 +1801,12 @@ impl FloatWrapper<FLOAT> {
     /// Create a new [`FloatWrapper`].
     #[inline(always)]
     #[must_use]
-    pub(crate) const fn const_new(value: FLOAT) -> Self {
+    pub const fn new_const(value: FLOAT) -> Self {
         Self(value)
     }
 }
 
-/// _(INTERNALS)_ An expression sub-tree.
+/// _(internals)_ An expression sub-tree.
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -1759,10 +1870,10 @@ pub enum Expr {
     Stmt(Box<StmtBlock>),
     /// func `(` expr `,` ... `)`
     FnCall(Box<FnCallExpr>, Position),
-    /// lhs `.` rhs
-    Dot(Box<BinaryExpr>, Position),
-    /// expr `[` expr `]`
-    Index(Box<BinaryExpr>, Position),
+    /// lhs `.` rhs - bool variable is a dummy
+    Dot(Box<BinaryExpr>, bool, Position),
+    /// expr `[` expr `]` - boolean indicates whether the dotting/indexing chain stops
+    Index(Box<BinaryExpr>, bool, Position),
     /// lhs `&&` rhs
     And(Box<BinaryExpr>, Position),
     /// lhs `||` rhs
@@ -1839,10 +1950,18 @@ impl fmt::Debug for Expr {
                 }
                 ff.finish()
             }
-            Self::Dot(x, pos) | Self::Index(x, pos) | Self::And(x, pos) | Self::Or(x, pos) => {
+            Self::Index(x, term, pos) => {
+                display_pos = *pos;
+
+                f.debug_struct("Index")
+                    .field("lhs", &x.lhs)
+                    .field("rhs", &x.rhs)
+                    .field("terminate", term)
+                    .finish()
+            }
+            Self::Dot(x, _, pos) | Self::And(x, pos) | Self::Or(x, pos) => {
                 let op_name = match self {
-                    Self::Dot(_, _) => "Dot",
-                    Self::Index(_, _) => "Index",
+                    Self::Dot(_, _, _) => "Dot",
                     Self::And(_, _) => "And",
                     Self::Or(_, _) => "Or",
                     _ => unreachable!(),
@@ -1915,9 +2034,18 @@ impl Expr {
             Union::Char(c, _, _) => Self::CharConstant(c, pos),
             Union::Int(i, _, _) => Self::IntegerConstant(i, pos),
 
+            #[cfg(feature = "decimal")]
+            Union::Decimal(value, _, _) => Self::DynamicConstant(Box::new((*value).into()), pos),
+
             #[cfg(not(feature = "no_float"))]
             Union::Float(f, _, _) => Self::FloatConstant(f, pos),
 
+            #[cfg(not(feature = "no_index"))]
+            Union::Array(a, _, _) => Self::DynamicConstant(Box::new((*a).into()), pos),
+
+            #[cfg(not(feature = "no_object"))]
+            Union::Map(m, _, _) => Self::DynamicConstant(Box::new((*m).into()), pos),
+
             _ => Self::DynamicConstant(value.into(), pos),
         }
     }
@@ -1964,7 +2092,7 @@ impl Expr {
             Self::Property(x) => (x.2).1,
             Self::Stmt(x) => x.1,
 
-            Self::And(x, _) | Self::Or(x, _) | Self::Dot(x, _) | Self::Index(x, _) => {
+            Self::And(x, _) | Self::Or(x, _) | Self::Dot(x, _, _) | Self::Index(x, _, _) => {
                 x.lhs.position()
             }
         }
@@ -1986,8 +2114,8 @@ impl Expr {
             | Self::Map(_, pos)
             | Self::And(_, pos)
             | Self::Or(_, pos)
-            | Self::Dot(_, pos)
-            | Self::Index(_, pos)
+            | Self::Dot(_, _, pos)
+            | Self::Index(_, _, pos)
             | Self::Variable(_, pos, _)
             | Self::Stack(_, pos)
             | Self::FnCall(_, pos)
@@ -2024,10 +2152,7 @@ impl Expr {
     #[inline(always)]
     #[must_use]
     pub const fn is_unit(&self) -> bool {
-        match self {
-            Self::Unit(_) => true,
-            _ => false,
-        }
+        matches!(self, Self::Unit(_))
     }
     /// Is the expression a constant?
     #[inline]
@@ -2078,8 +2203,8 @@ impl Expr {
             | Self::InterpolatedString(_, _)
             | Self::FnCall(_, _)
             | Self::Stmt(_)
-            | Self::Dot(_, _)
-            | Self::Index(_, _)
+            | Self::Dot(_, _, _)
+            | Self::Index(_, _, _)
             | Self::Array(_, _)
             | Self::Map(_, _) => match token {
                 #[cfg(not(feature = "no_index"))]
@@ -2144,7 +2269,7 @@ impl Expr {
                     }
                 }
             }
-            Self::Index(x, _) | Self::Dot(x, _) | Expr::And(x, _) | Expr::Or(x, _) => {
+            Self::Index(x, _, _) | Self::Dot(x, _, _) | Expr::And(x, _) | Expr::Or(x, _) => {
                 if !x.lhs.walk(path, on_node) {
                     return false;
                 }
