@@ -48,6 +48,8 @@ fn print_help() {
     println!("scope      => print all variables in the scope");
     #[cfg(feature = "metadata")]
     println!("functions  => print all functions defined");
+    #[cfg(feature = "metadata")]
+    println!("json       => output all functions in JSON format");
     println!("ast        => print the last AST (optimized)");
     println!("astu       => print the last raw, un-optimized AST");
     println!(r"end a line with '\' to continue to the next line.");
@@ -76,7 +78,12 @@ fn main() {
                     eprintln!("Error script file path: {}\n{}", filename, err);
                     exit(1);
                 }
-                Ok(f) => f,
+                Ok(f) => {
+                    match f.strip_prefix(std::env::current_dir().unwrap().canonicalize().unwrap()) {
+                        Ok(f) => f.into(),
+                        _ => f,
+                    }
+                }
             };
 
             contents.clear();
@@ -207,7 +214,7 @@ fn main() {
                     .enumerate()
                     .for_each(|(i, (name, constant, value))| {
                         #[cfg(not(feature = "no_closure"))]
-                        let value_is_shared = if value.is_shared() { " (shared" } else { "" };
+                        let value_is_shared = if value.is_shared() { " (shared)" } else { "" };
                         #[cfg(feature = "no_closure")]
                         let value_is_shared = "";
 
@@ -247,15 +254,16 @@ fn main() {
                 println!();
                 continue;
             }
-            // "json" => {
-            //     println!(
-            //         "{}",
-            //         engine
-            //             .gen_fn_metadata_with_ast_to_json(&main_ast, true)
-            //             .unwrap()
-            //     );
-            //     continue;
-            // }
+            #[cfg(feature = "metadata")]
+            "json" => {
+                println!(
+                    "{}",
+                    engine
+                        .gen_fn_metadata_with_ast_to_json(&main_ast, true)
+                        .unwrap()
+                );
+                continue;
+            }
             _ => (),
         }
 
